from __future__ import annotations

import os
import sys
from importlib import import_module

from autopub.plugins import AutopubPlugin


class PluginNotFoundError(Exception):
    ...


def _is_autopub_plugin(obj: object) -> bool:
    if (
        isinstance(obj, type)
        and issubclass(obj, AutopubPlugin)
        and obj is not AutopubPlugin
    ):
        return True

    return False


def _find_plugin(plugin: str) -> type[AutopubPlugin] | None:
    module_name = plugin
    symbol_name: str | None = None

    if ":" in plugin:
        module_name, symbol_name = plugin.split(":", 1)

    try:
        module = import_module(module_name)
    except ModuleNotFoundError:
        return None

    if symbol_name:
        obj = getattr(module, symbol_name)

        assert _is_autopub_plugin(obj)
        return obj
    else:
        symbols = {
            key: value
            for key, value in module.__dict__.items()
            if not key.startswith("__")
        }

        if "__all__" in module.__dict__:
            symbols = {
                name: symbol
                for name, symbol in symbols.items()
                if name in module.__dict__["__all__"]
            }

        for obj in symbols.values():
            if _is_autopub_plugin(obj):
                return obj

    return None


def load_plugins(names: list[str]) -> list[type[AutopubPlugin]]:
    sys.path.append(os.getcwd())

    plugins: list[type] = []

    for plugin_name in names:
        plugin_class = _find_plugin(f"{plugin_name}")

        if plugin_class is None:
            plugin_class = _find_plugin(f"autopub.plugins.{plugin_name}")

        if plugin_class is None:
            raise PluginNotFoundError(f"Could not find plugin {plugin_name}")

        plugins.append(plugin_class)

    return plugins



from __future__ import annotations

from datetime import date
from pathlib import Path

from autopub.plugins import AutopubPlugin
from autopub.types import ReleaseInfo

# TODO: from config
CHANGELOG_HEADER = "========="
VERSION_HEADER = "-"


class UpdateChangelogPlugin(AutopubPlugin):
    @property
    def changelog_file(self) -> Path:
        return Path("CHANGELOG.md")

    def post_prepare(self, release_info: ReleaseInfo) -> None:
        if not self.changelog_file.exists():
            self.changelog_file.write_text(f"CHANGELOG\n{CHANGELOG_HEADER}\n\n")

        current_date = date.today().strftime("%Y-%m-%d")

        old_changelog_data = ""
        header = ""

        lines = self.changelog_file.read_text().splitlines()

        for index, line in enumerate(lines):
            if CHANGELOG_HEADER != line.strip():
                continue

            old_changelog_data = lines[index + 1 :]
            header = lines[: index + 1]
            break

        new_version = release_info.version

        assert new_version is not None

        with self.changelog_file.open("w") as f:
            f.write("\n".join(header))
            f.write("\n")

            new_version_header = f"{new_version} - {current_date}"

            f.write(f"\n{new_version_header}\n")
            f.write(f"{VERSION_HEADER * len(new_version_header)}\n\n")
            f.write(release_info.release_notes)

            for line in release_info.additional_release_notes:
                f.write(f"\n\n{line}")

            f.write("\n")
            f.write("\n".join(old_changelog_data))



from __future__ import annotations

from typing import Any

from autopub.plugins import AutopubPackageManagerPlugin, AutopubPlugin


class UvPlugin(AutopubPlugin, AutopubPackageManagerPlugin):
    def build(self) -> None:
        self.run_command(["uv", "build"])

    def publish(self, repository: str | None = None, **kwargs: Any) -> None:
        additional_args: list[str] = []

        if repository:
            raise ValueError("Not yet implemented")

        if publish_url := kwargs.get("publish_url"):
            additional_args.append("--publish-url")
            additional_args.append(publish_url)

        if username := kwargs.get("username"):
            additional_args.append("--username")
            additional_args.append(username)

        if password := kwargs.get("password"):
            additional_args.append("--password")
            additional_args.append(password)

        if token := kwargs.get("token"):
            additional_args.append("--token")
            additional_args.append(token)

        self.run_command(["uv", "publish", *additional_args])
